Stan Math Library: stan/math/prim/mat/prob/lkj_corr_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

lkj_corr_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_MAT_PROB_LKJ_CORR_LOG_HPP
2 #define STAN_MATH_PRIM_MAT_PROB_LKJ_CORR_LOG_HPP
3
4 #include <stan/math/prim/mat/err/check_corr_matrix.hpp>
5 #include <stan/math/prim/scal/err/check_finite.hpp>
6 #include <stan/math/prim/scal/err/check_positive.hpp>
7 #include <stan/math/prim/scal/fun/constants.hpp>
8 #include <stan/math/prim/scal/meta/include_summand.hpp>
9 #include <stan/math/prim/scal/fun/lgamma.hpp>
10 #include <stan/math/prim/mat/fun/factor_cov_matrix.hpp>
11 #include <stan/math/prim/mat/fun/factor_U.hpp>
12 #include <stan/math/prim/mat/fun/read_corr_L.hpp>
13 #include <stan/math/prim/mat/fun/read_corr_matrix.hpp>
14 #include <stan/math/prim/mat/fun/read_cov_L.hpp>
15 #include <stan/math/prim/mat/fun/read_cov_matrix.hpp>
16 #include <stan/math/prim/mat/fun/make_nu.hpp>
17 #include <stan/math/prim/scal/fun/identity_constrain.hpp>
18 #include <stan/math/prim/scal/fun/identity_free.hpp>
19 #include <stan/math/prim/scal/fun/positive_constrain.hpp>
20 #include <stan/math/prim/scal/fun/positive_free.hpp>
21 #include <stan/math/prim/scal/fun/lb_constrain.hpp>
22 #include <stan/math/prim/scal/fun/lb_free.hpp>
23 #include <stan/math/prim/scal/fun/ub_constrain.hpp>
24 #include <stan/math/prim/scal/fun/ub_free.hpp>
25 #include <stan/math/prim/scal/fun/lub_constrain.hpp>
26 #include <stan/math/prim/scal/fun/lub_free.hpp>
27 #include <stan/math/prim/scal/fun/prob_constrain.hpp>
28 #include <stan/math/prim/scal/fun/prob_free.hpp>
29 #include <stan/math/prim/scal/fun/corr_constrain.hpp>
30 #include <stan/math/prim/scal/fun/corr_free.hpp>
31 #include <stan/math/prim/mat/fun/simplex_constrain.hpp>
32 #include <stan/math/prim/mat/fun/simplex_free.hpp>
33 #include <stan/math/prim/mat/fun/ordered_constrain.hpp>
34 #include <stan/math/prim/mat/fun/ordered_free.hpp>
35 #include <stan/math/prim/mat/fun/positive_ordered_constrain.hpp>
36 #include <stan/math/prim/mat/fun/positive_ordered_free.hpp>
37 #include <stan/math/prim/mat/fun/cholesky_factor_constrain.hpp>
38 #include <stan/math/prim/mat/fun/cholesky_factor_free.hpp>
39 #include <stan/math/prim/mat/fun/cholesky_corr_constrain.hpp>
40 #include <stan/math/prim/mat/fun/cholesky_corr_free.hpp>
41 #include <stan/math/prim/mat/fun/corr_matrix_constrain.hpp>
42 #include <stan/math/prim/mat/fun/corr_matrix_free.hpp>
43 #include <stan/math/prim/mat/fun/cov_matrix_constrain.hpp>
44 #include <stan/math/prim/mat/fun/cov_matrix_free.hpp>
45 #include <stan/math/prim/mat/fun/cov_matrix_constrain_lkj.hpp>
46 #include <stan/math/prim/mat/fun/cov_matrix_free_lkj.hpp>
47 #include <stan/math/prim/mat/fun/sum.hpp>
48
49 namespace stan {
50 namespace math {
51
52 template <typename T_shape>
53 T_shape do_lkj_constant(const T_shape& eta, const unsigned int& K) {
54 using stan::math::sum;
55 using stan::math::lgamma;
56
57 // Lewandowski, Kurowicka, and Joe (2009) theorem 5
58 T_shape constant;
59 const int Km1 = K - 1;
60 if (eta == 1.0) {
61 // C++ integer division is appropriate in this block
62 Eigen::VectorXd numerator(Km1 / 2);
63 for (int k = 1; k <= numerator.rows(); k++)
64 numerator(k-1) = lgamma(2 * k);
65 constant = sum(numerator);
66 if ( (K % 2) == 1 )
67 constant += 0.25 * (K * K - 1) * LOG_PI
68 - 0.25 * (Km1 * Km1) * LOG_TWO - Km1 * lgamma((K + 1) / 2);
69 else
70 constant += 0.25 * K * (K - 2) * LOG_PI
71 + 0.25 * (3 * K * K - 4 * K) * LOG_TWO
72 + K * lgamma(K / 2) - Km1 * lgamma(K);
73 } else {
74 constant = -Km1 * lgamma(eta + 0.5 * Km1);
75 for (int k = 1; k <= Km1; k++)
76 constant += 0.5 * k * LOG_PI + lgamma(eta + 0.5 * (Km1 - k));
77 }
78 return constant;
79 }
80
81 // LKJ_Corr(y|eta) [ y correlation matrix (not covariance matrix)
82 // eta > 0; eta == 1 <-> uniform]
83 template <bool propto,
84 typename T_y, typename T_shape>
85 typename boost::math::tools::promote_args<T_y, T_shape>::type
86 lkj_corr_log(const Eigen::Matrix<T_y, Eigen::Dynamic, Eigen::Dynamic>& y,
87 const T_shape& eta) {
88 static const char* function("stan::math::lkj_corr_log");
89
90 using stan::math::check_positive;
91 using stan::math::check_corr_matrix;
92 using stan::math::sum;
93 using boost::math::tools::promote_args;
94
95 typename promote_args<T_y, T_shape>::type lp(0.0);
96 check_positive(function, "Shape parameter", eta);
97 check_corr_matrix(function, "Correlation matrix", y);
98
99 const unsigned int K = y.rows();
100 if (K == 0)
101 return 0.0;
102
103 if (include_summand<propto, T_shape>::value)
104 lp += do_lkj_constant(eta, K);
105
106 if ( (eta == 1.0) &&
107 stan::is_constant<typename stan::scalar_type<T_shape> >::value )
108 return lp;
109
110 if (!include_summand<propto, T_y, T_shape>::value)
111 return lp;
112
113 Eigen::Matrix<T_y, Eigen::Dynamic, 1> values =
114 y.ldlt().vectorD().array().log().matrix();
115 lp += (eta - 1.0) * sum(values);
116 return lp;
117 }
118
119 template <typename T_y, typename T_shape>
120 inline
121 typename boost::math::tools::promote_args<T_y, T_shape>::type
122 lkj_corr_log(const Eigen::Matrix<T_y, Eigen::Dynamic, Eigen::Dynamic>& y,
123 const T_shape& eta) {
124 return lkj_corr_log<false>(y, eta);
125 }
126
127 }
128 }
129 #endif
ub_constrain.hpp
stan::math::sum
fvar< T > sum(const std::vector< fvar< T > > &m)
Return the sum of the entries of the specified standard vector.
Definition: sum.hpp:20
factor_U.hpp
stan::is_constant
Metaprogramming struct to detect whether a given type is constant in the mathematical sense (not the ...
Definition: is_constant.hpp:22
lgamma.hpp
stan::scalar_type
Metaprogram structure to determine the base scalar type of a template argument.
Definition: scalar_type.hpp:34
stan::math::lkj_corr_log
boost::math::tools::promote_args< T_y, T_shape >::type lkj_corr_log(const Eigen::Matrix< T_y, Eigen::Dynamic, Eigen::Dynamic > &y, const T_shape &eta)
Definition: lkj_corr_log.hpp:86
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
cov_matrix_free_lkj.hpp
positive_free.hpp
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::LOG_PI
const double LOG_PI
Definition: constants.hpp:170
lb_free.hpp
cov_matrix_free.hpp
corr_matrix_constrain.hpp
cholesky_factor_free.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::math::LOG_TWO
const double LOG_TWO
Definition: constants.hpp:177
include_summand.hpp
corr_constrain.hpp
prob_free.hpp
ordered_constrain.hpp
check_corr_matrix.hpp
cov_matrix_constrain_lkj.hpp
read_cov_L.hpp
simplex_free.hpp
cholesky_corr_constrain.hpp
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
identity_free.hpp
simplex_constrain.hpp
prob_constrain.hpp
cholesky_factor_constrain.hpp
stan::math::check_corr_matrix
bool check_corr_matrix(const char *function, const char *name, const Eigen::Matrix< T_y, Eigen::Dynamic, Eigen::Dynamic > &y)
Return true if the specified matrix is a valid correlation matrix.
Definition: check_corr_matrix.hpp:45
factor_cov_matrix.hpp
ub_free.hpp
cholesky_corr_free.hpp
constants.hpp
check_finite.hpp
cov_matrix_constrain.hpp
identity_constrain.hpp
sum.hpp
positive_ordered_constrain.hpp
positive_constrain.hpp
lub_constrain.hpp
read_cov_matrix.hpp
read_corr_matrix.hpp
stan::math::do_lkj_constant
T_shape do_lkj_constant(const T_shape &eta, const unsigned int &K)
Definition: lkj_corr_log.hpp:53
read_corr_L.hpp
lub_free.hpp
positive_ordered_free.hpp
corr_matrix_free.hpp
make_nu.hpp
lb_constrain.hpp
ordered_free.hpp
corr_free.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.